/*
 * UndoRedoTest.java
 *
 * Created on January 20, 2006, 4:24 PM by Matthew Ford
 *
Copyright (c) 2006 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.
 
Redistribution of the source of this package, with or without
modification, is NOT permitted.
 
Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.
 
2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.
 
3. Neither the name of the author nor Forward Computing and Control may be
   used to endorse or promote products derived from this software
   without specific prior written permission.
 
4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment:
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd.
    (http://www.forward.com.au)."
 
 
THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package au.com.forward.test.undoRedo;

import au.com.forward.undoRedo.BranchDefaultMutableTreeNode;
import au.com.forward.undoRedo.IDisplayText;
import au.com.forward.undoRedo.IHaveDescription;
import au.com.forward.undoRedo.UndoRedo;
import au.com.forward.undoRedo.UndoRedoManager;
import au.com.forward.undoRedo.UndoRedoTreeNode;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.KeyStroke;
import javax.swing.event.DocumentEvent;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;

/**
 *
 * @author  Matthew Ford
 */
public class UndoRedoTest extends javax.swing.JPanel implements UndoableEditListener {
    
    private String description =
            " This is a simple test panel for the UndoRedo Manager.\n"+
            " Type some text into the text box below.\n"+
            " Use Ctrl+Z to undo and Ctrl+Y to redo the typing.\n"+
            " \n"+
            " To show the UndoRedo tree press Ctrl+Shift+Z\n"+
            " Select any node to redo/undo to that point.\n"+
            " \n"+
            " The description shown next to the UndoRedo tree is prepared on a background thread.\n"+
            " There is a one second delay in showing the description.\n"+
            " As you change nodes the background thread for the previous node is cancelled \n"+
            " and a new thread started for the newly selected node.\n"+
            " In a real application you would probably use a java.util.concurrent.ExecutorService for this\n"+
            " in conjunction with FutureTalker which is available from www.forward.com.au\n"+
            "\n"+
            "\n See the companion article at www.forward.com.au for further information on the UndoRedoManager."+
            " Copyright (c) 2006  Forward Computing and Control Pty. Ltd.\n"+
            " NSW Australia.  All rights reserved.\n"+
            "\n"+
            " Subject to the Licence below, use of this software is free for commerical use.\n"+
            " \n\n"+
            "UndoRedo Package\n"+
            "Copyright (c) 2005 Forward Computing and Control Pty. Ltd.\n"+
            "NSW. Australia,  www.forward.com.au\n"+
            "All rights reserved.\n"+
            "\n"+
            "Redistribution of the source of this package, with or without\n"+
            "modification, is NOT permitted.\n"+
            "\n"+
            "Redistribution of the compiled java .class files of this package, with or without\n"+
            "modification, are permitted provided that the following conditions\n"+
            "are met:\n"+
            "1. Redistributions in compiled java .class form must reproduce the above\n"+
            "   copyright notice, this list of conditions and the following disclaimer\n"+
            "   in the documentation and/or other materials provided with the distribution.\n"+
            "\n"+
            "2. If the software is modified, the compiled java .class distribution must\n"+
            "   include, in the documentation and/or other materials provided with the\n"+
            "   distribution, a statement that the software has been modified and\n"+
            "   identify the person who was responsable for the modifications.\n"+
            "\n"+
            "3. Neither the name of the author nor Forward Computing and Control may be\n"+
            "   used to endorse or promote products derived from this software\n"+
            "   without specific prior written permission.\n"+
            "\n"+
            "4.  All advertising materials mentioning features or use of this\n"+
            "    software must display the following acknowledgment:\n"+
            "    \"This product includes software developed by Matthew Ford\n"+
            "     and Forward Computing and Control Pty. Ltd.\n"+
            "    (http://www.forward.com.au).\"\n"+
            "\n"+
            "\n"+
            "THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR\n"+
            "IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES\n"+
            "OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.\n"+
            "IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,\n"+
            "INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT\n"+
            "NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,\n"+
            "DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY\n"+
            "THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT\n"+
            "(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF\n"+
            "THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n";
    
    
    /**
     * A dialog to display the undoRedo tree in
     */
    private JDialog undoRedoDialog;
    
    /**
     * This example's main frame
     */
    private static JFrame mainFrame;
    
    /**
     * The undoRedoManager
     */
    private UndoRedoManager undoRedoManager;
    
    /**
     * This test panel
     */
    private static UndoRedoTest panel;
    
    /** Creates new form UndoRedoTest */
    public UndoRedoTest() {
        initComponents();
        doSetup();
    }
    
    /**
     * Make sure the input Text field gets focus
     */
    public boolean requestFocusInWindow(){
        return  inputField.requestFocusInWindow();
    }
    
    /**
     * Do the other setup here after components initialized
     */
    protected void doSetup() {
        undoRedoManager = new UndoRedoManager();
        PlainDocument doc = (PlainDocument)inputField.getDocument();
        doc.addUndoableEditListener(this);
        descriptionTextArea.setText(description);
        undoRedoDialog = new JDialog(mainFrame,"UndoRedo Tree display",false);
        undoRedoDialog.add(undoRedoManager);
        undoRedoDialog.getRootPane().getContentPane().setPreferredSize(new Dimension(800,600));
        undoRedoDialog.pack();
        setKeyboardMappings();
    }
    
    /**
     * The  undo action
     */
    private Action undoAction =
            new AbstractAction("undoAction") {
        public void actionPerformed(ActionEvent evt) {
            undoRedoManager.undo();
        }
    };
    
    /**
     * The redo action
     */
    private Action redoAction =
            new AbstractAction("redoAction") {
        public void actionPerformed(ActionEvent evt) {
            undoRedoManager.redo();
        }
    };
    
    /**
     * The show undoRedoDialog action
     */
    private Action showUndoRedoDialogAction =
            new AbstractAction("showUndoRedoDialogAction") {
        public void actionPerformed(ActionEvent evt) {
            // center the dialog on the screen
            Dimension size = undoRedoDialog.getSize();
            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
            undoRedoDialog.setLocation((screen.width - size.width) / 2, (screen.height - size.height) / 2);
            undoRedoDialog.setVisible(true);
        }
    };
    
    /**
     * The hide undoRedoDialog action
     */
    private Action hideUndoRedoDialogAction =
            new AbstractAction("hideUndoRedoDialogAction") {
        public void actionPerformed(ActionEvent evt) {
            undoRedoDialog.setVisible(false);
        }
    };
    
    /**
     * Set the show and hide dialog keyboard mappings and
     * the usual undo/redo mappings.
     */
    protected void setKeyboardMappings() {
        // for test window
        InputMap im = new InputMap();
        im.put(KeyStroke.getKeyStroke("control shift Z"), showUndoRedoDialogAction.getValue(Action.NAME));
        im.put(KeyStroke.getKeyStroke("control Z"), undoAction.getValue(Action.NAME));
        im.put(KeyStroke.getKeyStroke("control Y"), redoAction.getValue(Action.NAME));
        
        ActionMap am = new ActionMap();
        am.put(showUndoRedoDialogAction.getValue(Action.NAME),showUndoRedoDialogAction);
        am.put(redoAction.getValue(Action.NAME),redoAction);
        am.put(undoAction.getValue(Action.NAME),undoAction);
        
        im.setParent(this.getInputMap());
        this.setInputMap(WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,im);
        
        am.setParent(this.getActionMap());
        this.setActionMap(am);
        
        // for dialog
        im = new InputMap();
        im.put(KeyStroke.getKeyStroke("ESCAPE"), hideUndoRedoDialogAction.getValue(Action.NAME));
        im.put(KeyStroke.getKeyStroke("control Z"), undoAction.getValue(Action.NAME));
        im.put(KeyStroke.getKeyStroke("control Y"), redoAction.getValue(Action.NAME));
        
        am = new ActionMap();
        am.put(hideUndoRedoDialogAction.getValue(Action.NAME),hideUndoRedoDialogAction);
        am.put(redoAction.getValue(Action.NAME),redoAction);
        am.put(undoAction.getValue(Action.NAME),undoAction);
        
        im.setParent(undoRedoDialog.getRootPane().getInputMap());
        undoRedoDialog.getRootPane().setInputMap(WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,im);
        
        am.setParent(undoRedoDialog.getRootPane().getActionMap());
        undoRedoDialog.getRootPane().setActionMap(am);
    }
    
    
    /**
     * Direct the text fields document undoableEvents here to be re-wrapped in an undo which
     * provides a desciption.
     */
    public void undoableEditHappened(UndoableEditEvent undoableEditEvent) {
        // if this is document edit event wrap it to show more information
        UndoableEdit undoableEdit = undoableEditEvent.getEdit();
        if (undoableEdit instanceof AbstractDocument.DefaultDocumentEvent) {
            undoableEditEvent = new UndoableEditEvent(undoableEditEvent.getSource()
            ,new DefaultDocumentUndoableEdit((AbstractDocument.DefaultDocumentEvent)undoableEdit) );
            // wrap it to show more info
        }
        undoRedoManager.undoableEditHappened(undoableEditEvent);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jScrollPane1 = new javax.swing.JScrollPane();
        descriptionTextArea = new javax.swing.JTextArea();
        inputField = new javax.swing.JTextField();

        setLayout(new java.awt.GridBagLayout());

        descriptionTextArea.setColumns(20);
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setLineWrap(true);
        descriptionTextArea.setRows(5);
        descriptionTextArea.setWrapStyleWord(true);
        descriptionTextArea.setMinimumSize(new java.awt.Dimension(200, 200));
        jScrollPane1.setViewportView(descriptionTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jScrollPane1, gridBagConstraints);

        inputField.setFocusCycleRoot(true);
        inputField.setVerifyInputWhenFocusTarget(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(9, 2, 7, 2);
        add(inputField, gridBagConstraints);

    }// </editor-fold>//GEN-END:initComponents
    
    
    /**
     * The main method.
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        // JFrame.setDefaultLookAndFeelDecorated(true);
        mainFrame = new JFrame();
        panel = new UndoRedoTest();
        mainFrame.setSize(600, 400);
        
        mainFrame.add(panel);
        mainFrame.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        mainFrame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        mainFrame.setTitle("UndoRedo Manager Test");
        // mainFrame.pack();
        mainFrame.setVisible(true);
        panel.requestFocusInWindow();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea descriptionTextArea;
    private javax.swing.JTextArea descriptionTextArea1;
    private javax.swing.JTextField inputField;
    private javax.swing.JTextField inputField1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    // End of variables declaration//GEN-END:variables
    
    
    
    /**
     * An UndoableEdit that provides a description of the edit action.
     * Used to wrap AbstractDocument.DefaultDocumentEvent events.
     */
    public class DefaultDocumentUndoableEdit implements UndoableEdit, IHaveDescription {
        
        private String current = "";  // the text now
        private AbstractDocument.DefaultDocumentEvent defaultDocumentEvent;
        private DocumentEvent.EventType type;
        private String deltaText = ""; // the text added/removed
        
        public DefaultDocumentUndoableEdit(AbstractDocument.DefaultDocumentEvent docEvent) {
            this.defaultDocumentEvent = docEvent;
            type = defaultDocumentEvent.getType();
            // difficult to get the full details of the edit because GapContent uses package protected UndoEdit classes
            // just save the current field value after edit
            try {
                current = docEvent.getDocument().getText(0,docEvent.getDocument().getLength());
                if (type != DocumentEvent.EventType.REMOVE) {
                    deltaText = docEvent.getDocument().getText(docEvent.getOffset(),docEvent.getLength());
                }
            } catch (BadLocationException bex) {
                current = "Error getting current text: "+bex.getMessage();
            }
        }
        
        public boolean isDead() {
            return (!canRedo()) && (!canUndo());
        }
        
        /**
         * Gets the Description of the UndoableEdit.
         * <br>A delay is added here to simulate some processing.
         * <br>{@link #cancelGetDescription()} is called when the selection
         * changes and the text is no longer needed.
         */
        volatile Thread loadThread = null;
         public void getDescription(final IDisplayText textDisplay, final UndoRedoTreeNode undoRedoTreeNode) {
            if (undoRedoTreeNode.getUndoableEdit() != this) {
                throw new IllegalArgumentException(" UndoRedoTreeNode argument is not the node for this UndoableEdit.");
            } 
            UndoRedo undoRedo = ((UndoRedo)undoRedoTreeNode.getUserObject());
            final boolean undoFlag = ((UndoRedo)undoRedoTreeNode.getUserObject()).isUndo();
            textDisplay.setText("Loading description...");
            loadThread = new Thread(new Runnable() {
                public void run() {
                    if (loadThread == null) {
                        // got interrupt befor we could start.
                        // add more tests like this one to stop the thread promptly on interrupt()
                        return;
                    }
                    try {
                        Thread.currentThread().sleep(1000);
                        if (isDead()) {
                            textDisplay.setText("This node is dead.  Text after REDO was\n"+current);
                            return;
                        }   
                        if (undoFlag) {
                            // not a redo node
                            if (canUndo()) {
                                // look down to the next node to see what the result will be.
                                // skip Branch nodes
                                UndoRedoTreeNode parent = (UndoRedoTreeNode)undoRedoTreeNode.getParent();
                                UndoRedoTreeNode nextUndo = (UndoRedoTreeNode)parent.getChildAfter(undoRedoTreeNode);
                                while((nextUndo != null) && (nextUndo instanceof BranchDefaultMutableTreeNode)) {
                                     nextUndo = (UndoRedoTreeNode)parent.getChildAfter(nextUndo);                                    
                                }
                                if (nextUndo != null) {
                                    DefaultDocumentUndoableEdit undoableEdit = (DefaultDocumentUndoableEdit)nextUndo.getUndoableEdit();
                                    textDisplay.setText("   After this UNDO the text will be\n"+undoableEdit.current);                                    
                                } else {
                                    // reached begining of undos
                                    textDisplay.setText("   After this UNDO the text restored\n to its initial value.");                                                                        
                                }
                            } else {
                                throw new IllegalStateException(" Found a redo UndoableEdit in an Undo treeNode.");
                            }
                        } else {
                            // a redo node these nodes always restore the text to the current value
                            textDisplay.setText("After this REDO the text will be\n"+current);
                        }
                    } catch (Throwable t) {
                        try {
                            textDisplay.setText("Error getting description.\n"+t.getMessage());
                        } catch(Throwable tt) {
                            // log this serious error somewhere...
                        }
                    }
                }
            });
            loadThread.start();
        }
        
        /**
         * Cancels getDescription
         * <br>This is called when the undo/redo tree selection changes
         * to cancel the previous getDescription.
         * <br>In this example both this method and getDescription() are called on the
         * Swing thread so cancelGetDescription() will always cancel last getDescription()
         */
        public void cancelGetDescription() {
            Thread tempLoadThread = loadThread;
            loadThread = null;
            if (tempLoadThread != null) {
                tempLoadThread.interrupt();
            }
        }
        
        /**
         * Provides a localized, human readable description of this edit.
         * Because the UndoableEdit returned by GapContent is package protected, limited information is available.
         */
        public String getPresentationName() {
            if (type == DocumentEvent.EventType.INSERT) {
                return ""+type+" '"+deltaText+"' at offset:"+defaultDocumentEvent.getOffset();
            } else if (type == DocumentEvent.EventType.REMOVE) {
                return ""+type+" "+defaultDocumentEvent.getLength()+" characters at offset:"+defaultDocumentEvent.getOffset();
            } else {
                // CHANGE
                return ""+type+" '"+deltaText+"' at offset:"+defaultDocumentEvent.getOffset();
            }
        }
        
        /**
         * Provides a localized, human readable description of the undoable
         * form of this edit, e.g. for use as an Undo menu item. Typically
         * derived from <code>getPresentationName</code>.
         */
        public String getUndoPresentationName() {
            return "Undo "+getPresentationName();
        }
        
        /**
         * Provides a localized, human readable description of the redoable
         * form of this edit, e.g. for use as a Redo menu item. Typically
         * derived from <code>getPresentationName</code>.
         */
        public String getRedoPresentationName() {
            return "Redo "+getPresentationName();
        }
        
        
        /**
         * Undo the edit that was made.
         */
        public void undo() throws CannotUndoException {
            defaultDocumentEvent.undo();
        }
        
        /**
         * Re-apply the edit, assuming that it has been undone.
         */
        public void redo() throws CannotRedoException {
            defaultDocumentEvent.redo();
        }
        
        /**
         * True if it is still possible to undo this operation.
         */
        public boolean canUndo() {
            return defaultDocumentEvent.canUndo();
        }
        
        /**
         * True if it is still possible to redo this operation.
         */
        public boolean canRedo() {
            return defaultDocumentEvent.canRedo();
        }
        
        /**
         * May be sent to inform an edit that it should no longer be
         * used. This is a useful hook for cleaning up state no longer
         * needed once undoing or redoing is impossible--for example,
         * deleting file resources used by objects that can no longer be
         * undeleted. <code>UndoManager</code> calls this before it dequeues edits.
         *
         * Note that this is a one-way operation. There is no "un-die"
         * method.
         *
         * @see CompoundEdit#die
         */
        public void die() {
            defaultDocumentEvent.die();
        }
        
        /**
         * This <code>UndoableEdit</code> should absorb <code>anEdit</code>
         * if it can. Returns true
         * if <code.anEdit</code> has been incorporated, false if it has not.
         *
         * <p>Typically the receiver is already in the queue of a
         * <code>UndoManager</code> (or other <code>UndoableEditListener</code>),
         * and is being given a chance to incorporate <code>anEdit</code>
         * rather than letting it be added to the queue in turn.</p>
         *
         * <p>If true is returned, from now on <code>anEdit</code> must return
         * false from <code>canUndo</code> and <code>canRedo</code>,
         * and must throw the appropriate exception on <code>undo</code> or
         * <code>redo</code>.</p>
         * @param anEdit the edit to be added
         */
        public boolean addEdit(UndoableEdit anEdit) {
            return defaultDocumentEvent.addEdit(anEdit);
        }
        
        /**
         * Returns true if this <code>UndoableEdit</code> should replace
         * <code>anEdit</code>. The receiver should incorporate
         * <code>anEdit</code>'s state before returning true.
         *
         * <p>This message is the opposite of addEdit--anEdit has typically
         * already been queued in a <code>UndoManager</code> (or other
         * UndoableEditListener), and the receiver is being given a chance
         * to take its place.</p>
         *
         * <p>If true is returned, from now on anEdit must return false from
         * canUndo() and canRedo(), and must throw the appropriate
         * exception on undo() or redo().</p>
         */
        public boolean replaceEdit(UndoableEdit anEdit) {
            return defaultDocumentEvent.replaceEdit(anEdit);
        }
        
        /**
         * Returns false if this edit is insignificant--for example one
         * that maintains the user's selection, but does not change any
         * model state. This status can be used by an
         * <code>UndoableEditListener</code>
         * (like UndoManager) when deciding which UndoableEdits to present
         * to the user as Undo/Redo options, and which to perform as side
         * effects of undoing or redoing other events.
         */
        public boolean isSignificant() {
            return defaultDocumentEvent.isSignificant();
        }
        
    }
    
}
