/*
 * BranchUndoRedo.java
 * Created on March 14, 2005, 4:47 AM by Matthew Ford
 *
Copyright (c) 2005 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.
 
Redistribution of the source of this package, with or without
modification, is NOT permitted.
 
Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.
 
2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.
 
3. Neither the name of the author nor Forward Computing and Control may be
   used to endorse or promote products derived from this software
   without specific prior written permission.
 
4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment:
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd.
    (http://www.forward.com.au)."
 
 
THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package au.com.forward.undoRedo;

import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.UndoableEdit;

/**
 * An UndoRedo node that starts a Redo Branch.
 * @author Matthew Ford
 */
public class BranchUndoRedo extends UndoRedo {
  
  static public final UndoableEdit BRANCH_UNDOABLE_EDIT = new BranchUndoableEdit();
  private String branchSuffix = "...";
          
  /**
   * Creates a new instance of BranchUndoRedo.
   *<br>The wrapped UndoableEdit is a dummy.
   */
  public BranchUndoRedo() {
    super(new UndoRedo(BRANCH_UNDOABLE_EDIT));
  }
  
  /**
   * Returns true if the obj is the complement of this BranchUndoRedo.
   * UndoRedo complements wrap the same UndoableEdit object but
   * have complementary undo settings. BranchUndoRedo should never have complements.
   * @param complement  the object to test
   * @return true if the object is the complement of this BranchUndoRedo
   */
  public boolean equalsComplement(UndoRedo complement) {
    if (!(complement instanceof BranchUndoRedo)) {
      return false;
    }
    if (edit != complement.edit) {  // never have complement of branch
      return false;
    }
    return true;
  }
  
  /**
   * Sets the string suffix used for Branch Node lables.
   * @param name the suffix to use.
   */
  public void setBranchSuffix(String name) {
    if (name == null) {
      throw new IllegalArgumentException("Suffix cannot be null.");
    }
    branchSuffix = name;
  }
  
  /**
   * Returns the current suffix.
   * @return the current suffix added to the Branch Node lables.
   */
  public String getBranchSuffix() {
    return UndoRedo.HTML_START+UndoRedo.escapeHTML(branchSuffix);
  }
  
  
  /**
   * Returns the string represenation of this BranchNode, includes the suffix.
   * @return the string representation of this BranchNode. Used for the Tree Node lable.
   */
  public String toString() {
    return UndoRedo.HTML_START+UndoRedo.escapeHTML(super.toString())
      +UndoRedo.escapeHTML(branchSuffix);
  }
  
  /**
   * Get the complement of this object, keep reference to edit.
   * Actually never have complement so just return clone.
   * @return the clone of this BranchUndoRedo node.
   */
  public UndoRedo getComplement() {
    UndoRedo undoRedo = null;
    try {
      undoRedo = (UndoRedo)clone();
    } catch (CloneNotSupportedException ce) {
      throw new RuntimeException("Clone not supported!!",ce);
    }
    return undoRedo;
  }
  
  
  /**
   * A dummy UndoableEdit for the BranchUndoRedo node to wrap.
   */
  static public class BranchUndoableEdit extends AbstractUndoableEdit {
    public static final String PresentationName = " Branch to ";
    
    private BranchUndoableEdit() {
    }
    
    public String getPresentationName() {
      return  PresentationName;
    }
    
    public String getUndoPresentationName() {
      String name = getPresentationName();
      return name;
    }
    
    public String getRedoPresentationName() {
      String name = getPresentationName();
      return name;
    }
  }

  
}


