/*
 * IHaveDescription.java
 *
 * Created on April 18, 2005, 9:38 PM
Copyright (c) 2005 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.
 
Redistribution of the source of this package, with or without
modification, is NOT permitted.
 
Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.
 
2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.
 
3. Neither the name of the author nor Forward Computing and Control may be
   used to endorse or promote products derived from this software
   without specific prior written permission.
 
4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment:
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd.
    (http://www.forward.com.au)."
 
 
THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package au.com.forward.undoRedo;

/**
 * This interface indicates that the UndoableEdit has a long
 * description that gives more information then the getPresentationName()
 * @author Matthew Ford
 */
public interface IHaveDescription {

  /**
   * Gets the Description of the UndoableEdit.
   * <br>If getting the description will take some time,
   * such as generating a diff for example. Then
   * display a short message and start a thread to call back
   * to the textDisplay.setText() when the description is ready.
   * <br>If the selection has changed before textDisplay.setText() is
   * called, then the text is discarded.
   * <br>{@link #cancelGetDescription()} is called when the selection
   * changes and the text is no longer needed.
   * <p>A possiable implementation is:-
   *<pre>
      volatile Thread loadThread = null;
      volatile boolean interrupted = false;
      public void getDescription(IDisplayText textDisplay, boolean ) {
          interrupted = false;
          textDisplay.setText("Loading description...");
          final IDisplayText display = textDisplay;
          Thread loadThread = new Thread(new Runnable() {
              public void run() {
                  if (interrupted) {
                      // got interrupt befor we could start.
                      // add more tests like this one to stop the thread promptly on interrupt()
                      return;
                  }
                  try {
                      // create description here checking interrupted every so often
                      display.setText(description);
                  } catch (Throwable t) {
                      try {
                          display.setText("Error getting description.\n"+t.getMessage());
                      } catch(Throwable tt) {
                          // log this serious error somewhere...
                      }
                  }
              }
          });
          loadThread.start();
      }
      
      public void cancelGetDescription() {
          interrupted = true;
          if (loadThread != null) {
              loadThread.interrupt();
          }
      }   
   *
   */
  public void getDescription(IDisplayText textDisplay, UndoRedoTreeNode undoRedoTreeNode);
  
  /**
   * Cancels {@link #getDescription}.
   * <br>This is called when the undo/redo tree selection changes
   * to cancel the previous getDescription.
   */
  public void cancelGetDescription();
}
