/*
 * UndoRedoTreeNode.java
 *
 * Created on January 22, 2006, 3:42 PM
 *
 *
Copyright (c) 2006 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.
 
Redistribution of the source of this package, with or without
modification, is NOT permitted.
 
Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.
 
2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.
 
3. Neither the name of the author nor Forward Computing and Control may be
   used to endorse or promote products derived from this software
   without specific prior written permission.
 
4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment:
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd.
    (http://www.forward.com.au)."
 
 
THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package au.com.forward.undoRedo;

import java.util.Enumeration;
import java.util.Vector;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.undo.UndoableEdit;

/**
 *
 * @author Matthew Ford
 */

public class UndoRedoTreeNode extends DefaultMutableTreeNode implements Cloneable {
    
    /** Creates a new instance of UndoRedoTreeNode */
    public UndoRedoTreeNode() {
    }
    /**
     * Creates a tree node with no parent, no children, but which allows
     * children, and initializes it with the specified user object.
     *
     * @param userObject an Object provided by the user that constitutes
     *                   the node's data
     */
    public UndoRedoTreeNode(Object userObject) {
        super(userObject, true);
    }
    
    /**
     * Get the undoableEdit held by this node
     * @return  the undoableEdit for this node or null if this node
     * does not hold one, i.e. the rootNode.
     */
    public UndoableEdit getUndoableEdit() {
        UndoableEdit rtn = null;
        if (userObject instanceof UndoRedo) {
            rtn = ((UndoRedo)userObject).getUndoableEdit();
        }
        return rtn;
    }
    
    /**
     * The string representation of this node and the tree rooted on it.
     * @return a string representation of the tree starting at this node
     */
    public String toFullString() {
        String msg = "";
        if (userObject instanceof String) {
            // root node
            msg += " Root Node \n";
        } else {
            msg += "UndoRedoTreeNode containing "+
                    ((UndoRedo)userObject).getPresentationName()+"\n";
        }
        if (children != null) {
            Enumeration e = children.elements();
            while (e.hasMoreElements()) {
                msg += ((UndoRedoTreeNode)e.nextElement()).toFullString();
            }
        }
        return msg;
    }
    
    /**
     * Clone the undo/redo tree structure.
     * @return deep clone of the undo/redo tree structure, but without cloning the nodes
     * themselves
     */
    public Object clone() {
        UndoRedoTreeNode treeNode = (UndoRedoTreeNode)super.clone();
        
        // leave parent as null
        // now fix up vector and branches
        if (children != null) {
            treeNode.children = new Vector(children.size());
            Enumeration e = children.elements();
            while (e.hasMoreElements()) {
                UndoRedoTreeNode child = (UndoRedoTreeNode)((UndoRedoTreeNode)e.nextElement()).clone();
                child.setParent(treeNode);
                treeNode.children.add(child);
            }
        } else {
            treeNode.children = new Vector();
        }
        return treeNode;
    }
    
}
